#include "driver.h"

struct ZORAN Zoran;
struct ZR36120 Zr36120;

int OpenZoran(void)
{
uint r;
int i;
int BusDevFun,PCon;
const byte TabSAA7110Init[0x3c]={ // from philips' pdf
				0x4c,0x3c,0x0d,0xef,0xbd,0xf0,0x00,0x00,
				0xf8,0xf8,0x60,0x60,0x00,0x06,0x18,0x90,
				0x00,0x2c,0x40,0x46,0x42,0x1a,0xff,0xda,
				0xf0,0x8b,0x00,0x00,0x00,0x00,0x00,0x00,
				0xd9,0x16,0x40,0x41,0x80,0x41,0x80,0x4f,
				0xfe,0x01,0xcf,0x0f,0x03,0x01,0x81,0x03,
				0x44,0x71,0x02,0x8c,0x03	};



//** Preparar Acceso Memoria Tarjeta **//
if(r=CambiarLimiteDS())	return(100+r);

//** Buscar Tarjeta Capturadora **//
if (ExistePCIBIOS()!=0)	return(110);

memset(&Zoran,0,sizeof(Zoran));
if (BuscarDisp(DEVICE_ZR36120, VENDOR_ZORAN,&Zoran.BusDevFun))	return(120);

if (LeerPCIDW(Zoran.BusDevFun, PCI_BASE_ADDRESS, &Zoran.DirBase))	return(130);

// Habilitamos al zoran para acceder a memoria y como maestro de bus.
if (LeerPCIB(Zoran.BusDevFun, PCI_COMMAND, &Zoran.Command)) return(140);

Zoran.Command|=(PCI_COMMAND_MEMORY|PCI_COMMAND_MASTER);
if (EscribirPCIB(Zoran.BusDevFun, PCI_COMMAND, Zoran.Command))	return(150);

if (EscribirPCIB(Zoran.BusDevFun, PCI_LATENCY, LATENCIA))	return(160);


//** Inicializar Tarjeta **//
// Reset Zoran
Zr36120.SysPCIControl.SoftReset=0;
EscribirDW(Zoran.DirBase+SYSPCICONTROL_ADR,*(dword*)(&Zr36120.SysPCIControl));

// UnReset Zoran
Zr36120.SysPCIControl.SoftReset=1;
EscribirDW(Zoran.DirBase+SYSPCICONTROL_ADR,*(dword*)(&Zr36120.SysPCIControl));

//** Reset Philips SAA7110 **//
Zr36120.SysPCIControl.GenPurDir=0x00;
EscribirDW(Zoran.DirBase+SYSPCICONTROL_ADR,*(dword*)(&Zr36120.SysPCIControl));
Zr36120.GenPurpose.GenPurIO =0x00;
EscribirDW(Zoran.DirBase+GENPURPOSE_ADR,*(dword*)(&Zr36120.GenPurpose));

//** Inicializar SAA7110 **//
//if (I2CRead(SAA7110R,-1,&Version,1)<0)	return(12);
if (I2CWrite(SAA7110W,0,TabSAA7110Init,sizeof(TabSAA7110Init))<0)	return(13);

//** Inicilizar Zoran por defecto **//
Zoran.bOpen=TRUE;

// Actualizar estructura Zr36120
for (i=0;i<=0x11;i++)
	*(((dword*)&Zr36120)+i)=LeerDW(Zoran.DirBase+i*4);

// Con Compatibilidad con Triton 82437FX
Zr36120.VideoControl.TriCom=1;
// Tamao del buffer de pixel interno (7 DWORD)
Zr36120.VideoControl.MinPix=0x7;
EscribirDW(Zoran.DirBase+VIDEOCONTROL_ADR,*(dword*)(&Zr36120.VideoControl));

// Parametros Fijos Iniciales
Zr36120.OutVideoControl.ExtFI=1;
Zr36120.OutVideoControl.TopField=0;
EscribirDW(Zoran.DirBase+OUTVIDEOCONTROL_ADR,*(dword*)(&Zr36120.OutVideoControl));
Zr36120.HVideoControl.HSPol=0;
EscribirDW(Zoran.DirBase+HVIDEOCONTROL_ADR,*(dword*)(&Zr36120.HVideoControl));
Zr36120.VVideoControl.VSPol=1;
EscribirDW(Zoran.DirBase+VVIDEOCONTROL_ADR,*(dword*)(&Zr36120.VVideoControl));

 return 0;
}

int Config(struct PARAM_MODIF Cambios)
{
int i;
int We,hcrop1,hcrop2;
int He,vcrop1,vcrop2;
byte TabVidInpAdr[8]={0x06,0x20,0x21,0x22,0x2c,0x30,0x31,0x21};
byte TabVidInp[3][8]={
	{0x80, 0x9a, 0x17, 0xb1, 0x13, 0x60, 0xb5, 0x14},	// SVIDEO
	{0x00, 0xd8, 0x17, 0x40, 0x03, 0x44, 0x75, 0x16},	// VIDEO1
	{0x00, 0xb8, 0x07, 0x91, 0x03, 0x60, 0xb5, 0x05}};	// VIDEO2
byte TabEstAdr[7]={0x21,0x06,0x0F,0x0C,0x0D,0x11,0x2e};
byte TabEst[3][7]={
	{0x16, 0x00, 0x40, 0x80, 0x86, 0x2c, 0x81},	// NTSC
	{0x16, 0x00, 0x10, 0x80, 0x86, 0x59, 0x9a},	// PAL
	{0x16, 0x00, 0x30, 0x80, 0x87, 0x59, 0x9a}};	// SECAM


//** Comprobar Apertura **//
if (Zoran.bOpen==FALSE)	return(200);

if (Zr36120.VideoControl.VidEn)
	{
	Zr36120.VideoControl.VidEn=0;
	EscribirDW(Zoran.DirBase+VIDEOCONTROL_ADR,*(dword*)&Zr36120.VideoControl);
	}


//** Verificar Parametros **//
if (Cambios.Ancho>768 || Cambios.Alto>576)	return(210);
if (Cambios.Profundidad>7)	return(211);
if (Cambios.Escalado>3)	return(212);
if (Cambios.VidIn>2)	return(213);
if (Cambios.Estandar>2)	return(214);


//** Configurar Tarjeta **//
//** Dimensionar Imagen **//
switch(Cambios.Profundidad)
	{
	case YUV422:
		{
		Zr36120.OutVideoControl.YUV2RGB=0;
		Zoran.CapPixelBytes=2;
		break;
		}
	case RGB888:
		{
		Zr36120.OutVideoControl.YUV2RGB=1;
		Zoran.CapPixelBytes=3;
		Zr36120.OutVideoControl.LittleEndian=1;
		Zr36120.OutVideoControl.Pack24=1;
		break;
		}
	case RGB555:
		{
		Zr36120.OutVideoControl.YUV2RGB=3;
		Zoran.CapPixelBytes=2;
		Zr36120.OutVideoControl.LittleEndian=1;
		break;
		}
	case RGB565:
		{
		Zr36120.OutVideoControl.YUV2RGB=2;
		Zoran.CapPixelBytes=2;
		Zr36120.OutVideoControl.LittleEndian=1;
		break;
		}
	case BGR888:
		{
		Zr36120.OutVideoControl.YUV2RGB=1;
		Zoran.CapPixelBytes=3;
		Zr36120.OutVideoControl.LittleEndian=0;
		Zr36120.OutVideoControl.Pack24=1;
		break;
		}
	case BGR555:
		{
		Zr36120.OutVideoControl.YUV2RGB=3;
		Zoran.CapPixelBytes=2;
		Zr36120.OutVideoControl.LittleEndian=0;
		break;
		}
	case BGR565:
		{
		Zr36120.OutVideoControl.YUV2RGB=2;
		Zoran.CapPixelBytes=2;
		Zr36120.OutVideoControl.LittleEndian=0;
		break;
		}
	case RGB8888:
		{
		Zr36120.OutVideoControl.YUV2RGB=1;
		Zoran.CapPixelBytes=4;
		Zr36120.OutVideoControl.LittleEndian=1;
		Zr36120.OutVideoControl.Pack24=0;
		break;
		}
	default: return(220);
	}
Zr36120.OutVideoControl.DupFld=0;
Zr36120.OutVideoControl.HFilter=0;
Zr36120.OutVideoControl.ErrDif=0;
Zr36120.OutVideoControl.DispMod=1;
EscribirDW(Zoran.DirBase+OUTVIDEOCONTROL_ADR,*(dword*)&Zr36120.OutVideoControl);
Zoran.CapScanLineSize=Zoran.CapPixelBytes*Cambios.Ancho;

//** Configurar Entrada **//
for (i=0;i<8;i++)		// Linea de Entrada
	if (I2CWrite(SAA7110W,TabVidInpAdr[i],&TabVidInp[Cambios.VidIn][i],1)<0)	return(230);

for (i=0;i<7;i++)		// Estandar de Video
	if (I2CWrite(SAA7110W,TabEstAdr[i],&TabEst[Cambios.Estandar][i],1)<0)	return(240);


//** Configurar Percepcion Visual **/
if (I2CWrite(SAA7110W,BRILLO_ADR,&Cambios.Brillo,1)<0)	return(250);
if (I2CWrite(SAA7110W,CONTRASTE_ADR,&Cambios.Contraste,1)<0)	return(251);
if (I2CWrite(SAA7110W,COLOR_ADR,&Cambios.Color,1)<0)	return(252);
if (I2CWrite(SAA7110W,SATURACION_ADR,&Cambios.Saturacion,1)<0)	return(253);
if (I2CWrite(SAA7110W,GANANCIAUV_ADR,&Cambios.GananciaUV,1)<0)	return(254);

//**

Zr36120.VideoControl.VidWinWid=Cambios.Ancho;
Zr36120.VideoControl.VidWinHt=Cambios.Alto/2;
EscribirDW(Zoran.DirBase+VIDEOCONTROL_ADR,*(dword*)&Zr36120.VideoControl);
Zr36120.StrideGrabControl.DispStride=Cambios.InterLineado;
EscribirDW(Zoran.DirBase+STRIDEGRABCONTROL_ADR,*(dword*)&Zr36120.StrideGrabControl);
Zoran.InterCampo=Zoran.CapScanLineSize+(Zr36120.StrideGrabControl.DispStride-Zoran.CapScanLineSize)/2;

if (Cambios.Escalado&0x1)
	{	// Escalado horizontal
	Zr36120.OutVideoControl.HorDcm=(768-Cambios.Ancho)/12;
	EscribirDW(Zoran.DirBase+OUTVIDEOCONTROL_ADR,*(dword*)&Zr36120.OutVideoControl);
	We=Cambios.Ancho*64/(64-Zr36120.OutVideoControl.HorDcm);
	hcrop1=2*abs((768-We)/4);
	hcrop2=768-We-hcrop1;
	Zr36120.HVideoControl.HStart=143+hcrop1;
	Zr36120.HVideoControl.HEnd=910-hcrop2;
	EscribirDW(Zoran.DirBase+HVIDEOCONTROL_ADR,*(dword*)&Zr36120.HVideoControl);
	}
else{  // No Escalado horizontal
	Zr36120.OutVideoControl.HorDcm=0;
	EscribirDW(Zoran.DirBase+OUTVIDEOCONTROL_ADR,*(dword*)&Zr36120.OutVideoControl);
	Zr36120.HVideoControl.HStart=(1054-Cambios.Ancho)/2;
	Zr36120.HVideoControl.HEnd=Zr36120.HVideoControl.HStart+Cambios.Ancho-1;
	EscribirDW(Zoran.DirBase+HVIDEOCONTROL_ADR,*(dword*)&Zr36120.HVideoControl);
	}

if (Cambios.Escalado&0x2)
	{	// Escalado Vertical
	Zr36120.OutVideoControl.VerDcm=(576-Cambios.Alto)/9;
	EscribirDW(Zoran.DirBase+OUTVIDEOCONTROL_ADR,*(dword*)&Zr36120.OutVideoControl);
	He=((Cambios.Alto/2)*64)/(64-Zr36120.OutVideoControl.VerDcm);
	vcrop1=(576/2-He)/2;
	vcrop2=(576/2-He-vcrop1);
	Zr36120.VVideoControl.VStart=17+vcrop1;
	Zr36120.VVideoControl.VEnd=304-vcrop2;
	EscribirDW(Zoran.DirBase+VVIDEOCONTROL_ADR,*(dword*)&Zr36120.VVideoControl);
	}
else {  // No Escalado Vertical
	Zr36120.OutVideoControl.VerDcm=0;
	EscribirDW(Zoran.DirBase+OUTVIDEOCONTROL_ADR,*(dword*)&Zr36120.OutVideoControl);
	Zr36120.VVideoControl.VStart=17+(576-Cambios.Alto)/4;
	Zr36120.VVideoControl.VEnd=Zr36120.VVideoControl.VStart+Cambios.Alto;
	EscribirDW(Zoran.DirBase+VVIDEOCONTROL_ADR,*(dword*)&Zr36120.VVideoControl);
	}

// Overlay
if (Cambios.Overlay==NULL)
	{
	Zr36120.OverlayControl.OvlEnable=0;
	EscribirDW(Zoran.DirBase+OVERLAYCONTROL_ADR,*(dword*)&Zr36120.OverlayControl);
	}
else
	{
	Zr36120.OverlayControl.OvlEnable=1;
	Zr36120.OverlayControl.MaskStride=Cambios.Ancho/(8*4);
	EscribirDW(Zoran.DirBase+OVERLAYCONTROL_ADR,*(dword*)&Zr36120.OverlayControl);
	Zr36120.AdrTopVideoMaskMap=Cambios.Overlay;
	EscribirDW(Zoran.DirBase+ADRTOPVIDEOMASKMAP_ADR,*(dword*)&Zr36120.AdrTopVideoMaskMap);
	Zr36120.AdrBottomVideoMaskMap=Cambios.Overlay+Cambios.Ancho/8;
	EscribirDW(Zoran.DirBase+ADRBOTTOMVIDEOMASKMAP_ADR,*(dword*)&Zr36120.AdrBottomVideoMaskMap);
	}

return 0;
}

/************************************************************************/

int Capture(dword pFrameBuffer, word CapMode)
{
/* Comprobar Apertura */
if (Zoran.bOpen==FALSE)	return(300);

Zr36120.VideoControl.VidEn=0;
EscribirDW(Zoran.DirBase+VIDEOCONTROL_ADR,*(dword*)&Zr36120.VideoControl);

/* Configurar direccion de imagenes */
Zr36120.AdrTopVideo   =pFrameBuffer;
EscribirDW(Zoran.DirBase+ADRTOPVIDEO_ADR,*(dword*)&Zr36120.AdrTopVideo);
Zr36120.AdrBottomVideo=pFrameBuffer+Zoran.InterCampo;
EscribirDW(Zoran.DirBase+ADRBOTTOMVIDEO_ADR,*(dword*)&Zr36120.AdrBottomVideo);

/* Comenzar Captura */
switch (CapMode)
	{
	case GRAB_SIN:
	case GRAB_ASIN:
		{
		Zr36120.StrideGrabControl.SnapShot=1;
		EscribirDW(Zoran.DirBase+STRIDEGRABCONTROL_ADR,*(dword*)&Zr36120.StrideGrabControl);
		Zr36120.StrideGrabControl.FrameGrab=1;
		EscribirDW(Zoran.DirBase+STRIDEGRABCONTROL_ADR,*(dword*)&Zr36120.StrideGrabControl);
		break;
		}
	case CONTINUO:
		{
		Zr36120.StrideGrabControl.SnapShot=0;
		EscribirDW(Zoran.DirBase+STRIDEGRABCONTROL_ADR,*(dword*)&Zr36120.StrideGrabControl);
		Zr36120.StrideGrabControl.FrameGrab=0;
		EscribirDW(Zoran.DirBase+STRIDEGRABCONTROL_ADR,*(dword*)&Zr36120.StrideGrabControl);
		break;
		}
	case STOP: break;
	default: return(310);
	}

if (CapMode!=STOP)
	{
	Zr36120.VideoControl.VidEn=1;	// Let's go
	EscribirDW(Zoran.DirBase+VIDEOCONTROL_ADR,*(dword*)&Zr36120.VideoControl);
	}

if (CapMode==GRAB_SIN)	// Espera hasta que se transfiera la imagen
	while (Zr36120.StrideGrabControl.FrameGrab==1)
		*(((dword*)&Zr36120)+STRIDEGRABCONTROL_ADR/4)=LeerDW(Zoran.DirBase+STRIDEGRABCONTROL_ADR);


return 0;
}

int CloseZoran(void)
{
 // Comprobar Apertura
 if (Zoran.bOpen==FALSE) return 400;
 Zoran.bOpen=FALSE;
 // Reset Zoran
 Zr36120.SysPCIControl.SoftReset=0;
 EscribirDW(Zoran.DirBase+SYSPCICONTROL_ADR,*(dword*)(&Zr36120.SysPCIControl));

 if (RestablecerLimiteDS()) return 410;
 return 0;
}