#include "driver.h"

#define		I2CTIMEOUT	10
#define		I2CADRRETRY	10
#define MASK_SDA 2
#define MASK_SCL 1

/**********************************************************************************************************************/
BOOL	I2CGetSDA(void)
{
	uDelay();
	if (LeerDW(Zoran.DirBase+I2CCONTROL_ADR)&MASK_SDA)
		{
		Zr36120.I2CControl.SDA=1;
		return(TRUE);
		}
	else
		{
		Zr36120.I2CControl.SDA=0;
		return(FALSE);
		}
}

/**********************************************************************************************************************/
BOOL	I2CGetSCL(void)
{
	uDelay();
	if (LeerDW(Zoran.DirBase+I2CCONTROL_ADR)&MASK_SCL)
		{
		Zr36120.I2CControl.SCL=1;
		return(TRUE);
		}
	else
		{
		Zr36120.I2CControl.SCL=0;
		return(FALSE);
		}
}

/**********************************************************************************************************************/
void	I2CReleaseSDA(void)
{
	Zr36120.I2CControl.SDA=1;
	EscribirDW(Zoran.DirBase+I2CCONTROL_ADR,*(dword*)&Zr36120.I2CControl);
	uDelay();
}

/**********************************************************************************************************************/
BOOL	I2CSetSDA(BOOL bState)
{
	int			TimeOut;
	BOOL			bStill;
	bState=(bState!=0);   // asegurarse de que bState es 0 o 1.

	Zr36120.I2CControl.SDA=bState;
	EscribirDW(Zoran.DirBase+I2CCONTROL_ADR,*(dword*)&Zr36120.I2CControl);

	// Espera a que el I2C tenga bState.
	for (bStill=TRUE,TimeOut=I2CTIMEOUT;(TimeOut>0)&&(bStill);TimeOut--)
		{ uDelay(); if (I2CGetSDA()==bState) bStill=FALSE; }
	uDelay();

	return(bStill==FALSE);
}

/**********************************************************************************************************************/
BOOL	I2CSetSCL(BOOL bState)
{
	int			TimeOut;
	BOOL			bStill;
	bState=(bState!=0);

	Zr36120.I2CControl.SCL=bState;
	EscribirDW(Zoran.DirBase+I2CCONTROL_ADR,*(dword*)&Zr36120.I2CControl);
	for (bStill=TRUE,TimeOut=I2CTIMEOUT;(TimeOut>0)&&(bStill);TimeOut--)
		{ uDelay(); if (I2CGetSCL()==bState) bStill=FALSE; }
	uDelay();

	return(bStill==FALSE);
}

/**********************************************************************************************************************/
BOOL	I2CSendStart(void)
{
	/* SCL y SDA deben estar a 1 */
	if ((I2CGetSDA()==0)||(I2CGetSCL()==0))	return(FALSE);
	I2CSetSDA(0);
	uDelay();
	I2CSetSCL(0);
	uDelay();
	return(TRUE);
}


/**********************************************************************************************************************/
BOOL	I2CSendStop(void)
{
	/* SCL debe ser 0 */
	if ((I2CGetSCL()!=0) && (I2CSetSCL(0)==FALSE))	return(FALSE);
	I2CSetSDA(0);
	I2CSetSCL(1);
	uDelay();
	I2CSetSDA(1);
	uDelay();
	return(TRUE);
}


/**********************************************************************************************************************/
BOOL	I2CSendByte(uchar Byte)
{
	int			Mask;
	BOOL			bError;

	/* SCL debe ser 0, sin secuencia de start/stop */
	for (Mask=0x80;Mask!=0;Mask>>=1)
		{
		I2CSetSDA(Byte&Mask);
		if (I2CSetSCL(1)==FALSE)
			{
			I2CSetSDA(1);
			return(FALSE);
			}
		uDelay();
		if (((Byte&Mask)!=0)&&(I2CGetSDA()==0))	return(FALSE);
		I2CSetSCL(0);
		}
	I2CReleaseSDA();
	if (I2CSetSCL(1)==FALSE)	return(FALSE);
	uDelay();
	bError=I2CGetSDA();
	I2CSetSCL(0);
	uDelay();
	return(bError==0);
}

/**********************************************************************************************************************/
int	I2CGetByte(void)
{
	int			Mask;
	uchar			Byte;

	/* SCL debe ser 0, sin secuencia de start/stop */
	for (Byte=0,Mask=0x80;Mask!=0;Mask>>=1)
		{
		I2CSetSDA(1);
		if (I2CSetSCL(1)==FALSE)
			{
			I2CSetSDA(1);
			return(-1);
			}
		if (I2CGetSDA()) Byte+=Mask;
		I2CSetSCL(0);
		I2CSetSDA(0);
		}
	return(Byte);
}


/**********************************************************************************************************************/
int	I2CWrite(int I2CAdr,int I2CSubAdr,const char * TabByte,int NbMaxByte)
{
	int		IndexByte;
	int		Retry;

	for(Retry=0;TRUE;Retry++)
		{
		if (I2CSendStart())
			if (I2CSendByte(I2CAdr))
				{
				if (I2CSubAdr!=-1)
					{
					if (I2CSendByte(I2CSubAdr))	break;
					}
				else	break;
				}
		I2CSendStop();
		if (Retry>=I2CADRRETRY)	return(-1);
		}
	for(IndexByte=0;IndexByte<NbMaxByte;IndexByte++)
		if (I2CSendByte(TabByte[IndexByte])==FALSE)
			{
			I2CSendStop();
			return(-1);
			}
	I2CSendStop();
	return(IndexByte);
}